using System;
using System.Collections;
using System.Collections.Generic;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Linq;

namespace TroyaPrototypeV2
{
    public class ActivityContainer : MyBaseEntity
    {
        private ObservableCollection<Activity> _Activities;
        private ObservableCollection<ActivityType> _ActivityTypes;
        private ObservableCollection<ActivityGroup> _ActivityGroups;
        private IList<ActivityGroup> _IActivityGroups;
        private ObservableCollection<ActivityGroup> _FilteredActivityGroups;        
        private ObservableCollection<ActivityGroup> _RootActivityGroups;
        private ActivityType _CurrentType;
        

        public ActivityContainer()
        {
            ActivityTypes = new ObservableCollection<ActivityType>
            {
                new ActivityType{IDActivityType=1, ActivityTypeTerm="Operation"},
                new ActivityType{IDActivityType=2, ActivityTypeTerm="SubProcess"},
                new ActivityType{IDActivityType=3, ActivityTypeTerm="Process"},
                new ActivityType{IDActivityType=4, ActivityTypeTerm="All"}
            };          

            ActivityGroups = new ObservableCollection<ActivityGroup>
            {                                                
                new ActivityGroup{ IDActivityGroup = 1, IDActivityType = 1, GroupTerm = "I Operation Group", IDSuperGroup = 0, Template = "abc-001", HierarchyLevel = 1, Subgroup = false, OrderNumber = 1},
                new ActivityGroup{ IDActivityGroup = 2, IDActivityType = 1, GroupTerm = "I.I Operation SubGroup", IDSuperGroup = 1, Template = "abc-0011", HierarchyLevel = 2, Subgroup = true, OrderNumber = 2},
                new ActivityGroup{ IDActivityGroup = 3, IDActivityType = 1, GroupTerm = "I.II Operation SubGroup", IDSuperGroup = 1, Template = "abc-0012", HierarchyLevel = 2, Subgroup = true, OrderNumber = 3},
                new ActivityGroup{ IDActivityGroup = 4, IDActivityType = 1, GroupTerm = "I.II.I Operation SubGroup of Subgroup", IDSuperGroup = 3, Template = "abc-00121", HierarchyLevel = 3, Subgroup = true, OrderNumber = 4},
                              
                new ActivityGroup{ IDActivityGroup = 5, IDActivityType = 1, GroupTerm = "II Operation Group", IDSuperGroup = 0, Template = "abc-002", HierarchyLevel = 1, Subgroup = false, OrderNumber = 5},
                new ActivityGroup{ IDActivityGroup = 6, IDActivityType = 1, GroupTerm = "II.I Operation SubGroup", IDSuperGroup = 5, Template = "abc-0021", HierarchyLevel = 2, Subgroup = true, OrderNumber = 6},
                new ActivityGroup{ IDActivityGroup = 7, IDActivityType = 1, GroupTerm = "II.II Operation SubGroup", IDSuperGroup = 5, Template = "abc-0022", HierarchyLevel = 2, Subgroup = true, OrderNumber = 7},
               
                new ActivityGroup{ IDActivityGroup = 8, IDActivityType = 1, GroupTerm = "III Operation Group", IDSuperGroup = 0, Template = "abc-003", HierarchyLevel = 1, Subgroup = false, OrderNumber = 8},
                new ActivityGroup{ IDActivityGroup = 9, IDActivityType = 1, GroupTerm = "III.I Operation SubGroup", IDSuperGroup = 8, Template = "abc-0031", HierarchyLevel = 2, Subgroup = true, OrderNumber = 9},
                new ActivityGroup{ IDActivityGroup = 10, IDActivityType = 1, GroupTerm = "III.II Operation SubGroup", IDSuperGroup = 8, Template = "abc-0032", HierarchyLevel = 2, Subgroup = true, OrderNumber = 10},
                new ActivityGroup{ IDActivityGroup = 11, IDActivityType = 1, GroupTerm = "III.III Operation Subgroup", IDSuperGroup = 8, Template = "abc-0033", HierarchyLevel = 2, Subgroup = true, OrderNumber = 11},
               
                new ActivityGroup{ IDActivityGroup = 12, IDActivityType = 2, GroupTerm = "I SubProcess", IDSuperGroup = 0, Template = "abc-101", HierarchyLevel = 1, Subgroup = false, OrderNumber = 1},
                new ActivityGroup{ IDActivityGroup = 13, IDActivityType = 2, GroupTerm = "I.I SubProcess", IDSuperGroup = 12, Template = "abc-1011", HierarchyLevel = 2, Subgroup = true, OrderNumber = 2},
                new ActivityGroup{ IDActivityGroup = 14, IDActivityType = 2, GroupTerm = "I.II SubProcess", IDSuperGroup = 12, Template = "abc-1012", HierarchyLevel = 2, Subgroup = true, OrderNumber = 3},
                new ActivityGroup{ IDActivityGroup = 15, IDActivityType = 2, GroupTerm = "I.II.I SubProcess of subprocess", IDSuperGroup = 14, Template = "abc-10121", HierarchyLevel = 3, Subgroup = true, OrderNumber = 4},

                new ActivityGroup{ IDActivityGroup = 16, IDActivityType = 2, GroupTerm = "II SubProcess", IDSuperGroup = 0, Template = "abc-102", HierarchyLevel = 1, Subgroup = false, OrderNumber = 5},
                new ActivityGroup{ IDActivityGroup = 17, IDActivityType = 2, GroupTerm = "II.I SubProcess", IDSuperGroup = 16, Template = "abc-1021", HierarchyLevel = 2, Subgroup = true, OrderNumber = 6},
                new ActivityGroup{ IDActivityGroup = 18, IDActivityType = 2, GroupTerm = "II.II SubProcess", IDSuperGroup = 16, Template = "abc-1022", HierarchyLevel = 2, Subgroup = true, OrderNumber = 7},

                new ActivityGroup{ IDActivityGroup = 19, IDActivityType = 2, GroupTerm = "III SubProcess", IDSuperGroup = 0, Template = "abc-103", HierarchyLevel = 1, Subgroup = false, OrderNumber = 8},
                new ActivityGroup{ IDActivityGroup = 20, IDActivityType = 2, GroupTerm = "III.I SubProcess", IDSuperGroup = 19, Template = "abc-1031", HierarchyLevel = 2, Subgroup = true, OrderNumber = 9},
                new ActivityGroup{ IDActivityGroup = 21, IDActivityType = 2, GroupTerm = "III.II SubProcess", IDSuperGroup = 19, Template = "abc-1032", HierarchyLevel = 2, Subgroup = true, OrderNumber = 10},
                new ActivityGroup{ IDActivityGroup = 22, IDActivityType = 2, GroupTerm = "III.III SubProcess", IDSuperGroup = 19, Template = "abc-1033", HierarchyLevel = 2, Subgroup = true, OrderNumber = 11},
              
                new ActivityGroup{ IDActivityGroup = 23, IDActivityType = 3, GroupTerm = "I Process", IDSuperGroup = 0, Template = "abc-201", HierarchyLevel = 1, Subgroup = false, OrderNumber = 1},
                new ActivityGroup{ IDActivityGroup = 24, IDActivityType = 3, GroupTerm = "I.I Process", IDSuperGroup = 23, Template = "abc-2011", HierarchyLevel = 2, Subgroup = true, OrderNumber = 2},
                new ActivityGroup{ IDActivityGroup = 25, IDActivityType = 3, GroupTerm = "I.II Process", IDSuperGroup = 23, Template = "abc-2012", HierarchyLevel = 2, Subgroup = true, OrderNumber = 3},
                new ActivityGroup{ IDActivityGroup = 26, IDActivityType = 3, GroupTerm = "I.II.I Process", IDSuperGroup = 25, Template = "abc-20121", HierarchyLevel = 3, Subgroup = true, OrderNumber = 4},
               
                new ActivityGroup{ IDActivityGroup = 27, IDActivityType = 3, GroupTerm = "II Process", IDSuperGroup = 0, Template = "abc-202", HierarchyLevel = 1, Subgroup = false, OrderNumber = 5},
                new ActivityGroup{ IDActivityGroup = 28, IDActivityType = 3, GroupTerm = "II.I Process", IDSuperGroup = 27, Template = "abc-2021", HierarchyLevel = 2, Subgroup = true, OrderNumber = 6},
                new ActivityGroup{ IDActivityGroup = 29, IDActivityType = 3, GroupTerm = "II.II Process", IDSuperGroup = 27, Template = "abc-2022", HierarchyLevel = 2, Subgroup = true, OrderNumber = 7},

                new ActivityGroup{ IDActivityGroup = 30, IDActivityType = 3, GroupTerm = "III Process", IDSuperGroup = 0, Template = "abc-203", HierarchyLevel = 1, Subgroup = false, OrderNumber = 8},
                new ActivityGroup{ IDActivityGroup = 31, IDActivityType = 3, GroupTerm = "III.I Process", IDSuperGroup = 30, Template = "abc-2031", HierarchyLevel = 2, Subgroup = true, OrderNumber = 9},
                new ActivityGroup{ IDActivityGroup = 32, IDActivityType = 3, GroupTerm = "III.II Process", IDSuperGroup = 30, Template = "abc-2032", HierarchyLevel = 2, Subgroup = true, OrderNumber = 10},
                new ActivityGroup{ IDActivityGroup = 33, IDActivityType = 3, GroupTerm = "III.III Process", IDSuperGroup = 30, Template = "abc-2033", HierarchyLevel = 2, Subgroup = true, OrderNumber = 11}
                
            };
            
            try
            {
                FilteredActivityGroups = CreateWithList(ActivityGroups);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
            }
        }


        private static ObservableCollection<ActivityGroup> CreateWithList(ObservableCollection<ActivityGroup> _activityGroups)
        {               
            ObservableCollection<ActivityGroup> t_activityGroups = new ObservableCollection<ActivityGroup>();

            ActivityGroup t_temp = null;
            ObservableCollection<ActivityGroup> t_sub = null;
            foreach (ActivityGroup g in _activityGroups)
            {                
                if (g.IDSuperGroup == 0)
                {
                   if(t_temp != null)
                   {
                       t_temp.Subgroups = t_sub;
                       t_activityGroups.Add(t_temp);
                   }
                    
                    t_temp = g;
                    t_sub = new ObservableCollection<ActivityGroup>();                  
                }
                else
                {
                    if (g.IDSuperGroup == t_temp.IDActivityGroup)
                    {
                        try
                        {                            
                            t_sub.Add(g);
                        }
                        catch (Exception ex)
                        {
                            Console.Out.WriteLine(ex.Message);
                            Console.Out.WriteLine(ex.StackTrace);
                        }
                    }                   
                }                                                                            
            }

            return t_activityGroups;
        }

        public ObservableCollection<Activity> Activities
        {
            get
            {
                return _Activities;
            }

            set
            {
                if (_Activities != value)
                {
                    _Activities = value;
                    OnPropertyChanged("Activities");
                }
            }
        }

        public ObservableCollection<ActivityType> ActivityTypes
        {
            get
            {
                return _ActivityTypes;
            }

            set
            {
                if (_ActivityTypes != value)
                {
                    _ActivityTypes = value;
                    OnPropertyChanged("ActivityTypes");
                }
            }
        }


        public ObservableCollection<ActivityGroup> ActivityGroups
        {
            get
            {
                return _ActivityGroups;
            }

            set
            {
                if (_ActivityGroups != value)
                {
                    _ActivityGroups = value;
                    OnPropertyChanged("ActivityGroups");
                }
            }
        }

        public ActivityType CurrentType
        {
            get
            {
                return _CurrentType;
            }

            set
            {
                if (_CurrentType != value)
                {
                    _CurrentType = value;
                    OnPropertyChanged("CurrentType");
                    FilterActivityGroupsByActivityType();
                }
            }
        }

        public ObservableCollection<ActivityGroup> FilteredActivityGroups        
        {
            get
            {
                return _FilteredActivityGroups;
            }

            set
            {
                if (_FilteredActivityGroups != value)
                {
                    _FilteredActivityGroups = value;
                    OnPropertyChanged("FilteredActivityGroups");
                }
            }
        }

        public IList<ActivityGroup> IActivityGroups
        {
            get
            {
                return _IActivityGroups;
            }

            set
            {
                if (_IActivityGroups != value)
                {
                    _IActivityGroups = value;
                    OnPropertyChanged("IActivityGroups");
                }
            }
        }
        
        private void FilterActivityGroupsByActivityType()
        {
            if (CurrentType != null)
            {
                if (CurrentType.ActivityTypeTerm != "Svi zapisi")
                {
                    var activityGroups = ActivityGroups.Where(g => g.IDActivityType == CurrentType.IDActivityType);
                    FilteredActivityGroups = new ObservableCollection<ActivityGroup>(activityGroups);
                    IActivityGroups = FilteredActivityGroups.ToList<ActivityGroup>();
                }
                else
                {
                    FilteredActivityGroups = ActivityGroups;
                    IActivityGroups = FilteredActivityGroups.ToList<ActivityGroup>();
                }
            }
        }

        public IList<ActivityGroup> GetGroups()
        {
            return FilteredActivityGroups.ToList<ActivityGroup>();            
        }



      
    }
}
